@@ -3,69 +3,92 @@ use Moose ();
 use Carp;
 use Moose::Exporter;
 use URI::Escape;
+use DateTime::Format::Strptime;
 
 use namespace::autoclean;
 
 Moose::Exporter->setup_import_methods(
-    with_caller => [ 'base_url', 'twitter_api_method' ],
+    with_caller => [ qw/base_url authenticate datetime_parser twitter_api_method/ ],
 );
 
-my $post_request = sub {
-    my ($ua, $uri, $args) = @_;
-    return $ua->post($uri, $args);
-};
+sub base_url {
+    my ($caller, $name, %options) = @_;
 
-my $get_request = sub {
-    my ($ua, $uri, $args) = @_;
-    $uri->query_form($args);
-    return $ua->get($uri);
-};
+    Moose::Meta::Class->initialize($caller)->add_method(_base_url => sub { $_[1]->$name });
+}
 
-my $with_url_arg = sub {
-    my ($path, $args) = @_;
+# kludge: This is very transient!
+my $do_auth;
+sub authenticate { $do_auth = $_[1] }
 
-    if ( defined(my $id = delete $args->{id}) ) {
-        $path .= uri_escape($id);
-    }
-    else {
-        chop($path);
-    }
-    return $path;
-};
+# provide a default: we'll use the format of the REST API
+my $datetime_parser = DateTime::Format::Strptime->new(pattern => '%a %b %d %T %z %Y');
+sub datetime_parser { $datetime_parser = $_[1] }
 
 sub twitter_api_method {
-    my ($caller, $name, %options) = @_;
+    my $caller = shift;
+    my $name   = shift;
+    my %options = ( authenticate => $do_auth, datetime_parser => $datetime_parser, @_ );
 
     my $class = Moose::Meta::Class->initialize($caller);
 
     my ($arg_names, $path) = @options{qw/required path/};
     $arg_names = $options{params} if @$arg_names == 0 && @{$options{params}} == 1;
-    my $request = $options{method} eq 'POST' ? $post_request : $get_request;
-
-    my $modify_path = $path =~ s,/id$,/, ? $with_url_arg : sub { $_[0] };
 
     my $code = sub {
         my $self = shift;
 
-        my $args = {};
-        if ( ref $_[0] ) {
-            UNIVERSAL::isa($_[0], 'HASH') && @_ == 1 || croak "$name expected a single HASH ref argument";
-            $args = { %{shift()} }; # copy callers args since we may add ->{source}
-        }
-        elsif ( @_ ) {
-            @_ == @$arg_names || croak "$name expected @{[ scalar @$arg_names ]} args";
-            @{$args}{@$arg_names} = @_;
+        # copy callers args since we may add ->{source}
+        my $args = ref $_[-1] eq 'HASH' ? { %{pop @_} } : {};
+
+        croak sprintf "$name expected %d args", scalar @$arg_names if @_ > @$arg_names;
+
+        # promote positional args to named args
+        for ( my $i = 0; @_; ++$i ) {
+            my $param = $arg_names->[$i];
+            croak "duplicate param $param: both positional and named"
+                if exists $args->{$param};
+
+            $args->{$param} = shift;
         }
+
         $args->{source} ||= $self->source if $options{add_source};
 
-        my $local_path = $modify_path->($path, $args);
-        
+        # save synthetic arguments; don't pass them to Twitter
+        my $synthetic_args = $self->_extract_synthetic_args($args);
+
+        my $authenticate = exists $synthetic_args->{authenticate}
+                         ? $synthetic_args->{authenticate}
+                         : $options{authenticate}
+                         ;
+
+        # promote boolean parameters
+        for my $boolean_arg ( @{ $options{booleans} } ) {
+            if ( exists $args->{$boolean_arg} ) {
+                next if $args->{$boolean_arg} =~ /^true|false$/;
+                $args->{$boolean_arg} = $args->{$boolean_arg} ? 'true' : 'false';
+            }
+        }
+
+        # Workaround Twitter bug: any value passed for skip_user is treated as true.
+        # The only way to get 'false' is to not pass the skip_user at all.
+        delete $args->{skip_user} if exists $args->{skip_user} && $args->{skip_user} eq 'false';
+
+        # replace placeholder arguments
+        my $local_path = $path;
+        $local_path =~ s,/:id$,, unless exists $args->{id}; # remove optional trailing id
+        $local_path =~ s/:(\w+)/delete $args->{$1} or croak "required arg '$1' missing"/eg;
+
         my $uri = URI->new($caller->_base_url($self) . "/$local_path.json");
 
-        # upgrade params to UTF-8 so latin-1 literals can be handled as UTF-8 too
-        utf8::upgrade $_ for values %$args;
-
-        return $self->_parse_result($request->($self->ua, $uri, $args));
+        return $self->_json_request(
+            $options{method},
+            $uri,
+            $args,
+            $authenticate,
+            $synthetic_args,
+            $options{datetime_parser},
+        );
     };
 
     $class->add_method(
@@ -81,18 +104,12 @@ sub twitter_api_method {
     $class->add_method($_, $code) for @{$options{aliases} || []};
 }
 
-sub base_url {
-    my ($caller, $name, %options) = @_;
-    
-    Moose::Meta::Class->initialize($caller)->add_method(_base_url => sub { $_[1]->$name });
-}
-
-
 package Net::Twitter::Meta::Method;
 use Moose;
+use Carp;
 extends 'Moose::Meta::Method';
 
-use namespace::clean;
+use namespace::autoclean;
 
 has description => ( isa => 'Str', is => 'ro', required => 1 );
 has aliases     => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
@@ -103,8 +120,23 @@ has params      => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } )
 has required    => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
 has returns     => ( isa => 'Str', is => 'ro', predicate => 'has_returns' );
 has deprecated  => ( isa => 'Bool', is => 'ro', default => 0 );
+has booleans    => ( isa => 'ArrayRef[Str]', is => 'ro', default => sub { [] } );
+has authenticate => ( isa => 'Bool', is => 'ro', required => 1 );
+has datetime_parser => ( is => 'ro', required => 1 );
 
-sub new { shift->SUPER::wrap(@_) }
+# TODO: can MooseX::StrictConstructor be made to work here?
+my %valid_attribute_names = map { $_->init_arg => 1 }
+                            __PACKAGE__->meta->get_all_attributes;
+
+sub new {
+    my $class = shift;
+    my %args  = @_;
+
+    my @invalid_attributes = grep { !$valid_attribute_names{$_} } keys %args;
+    croak "unexpected argument(s): @invalid_attributes" if @invalid_attributes;
+
+    $class->SUPER::wrap(@_);
+}
 
 1;
 
@@ -161,6 +193,15 @@ Specifies, by name, the attribute which contains the base URL for the defined AP
 
 Defines a Twitter API method.  Valid arguments are:
 
+=item authenticate
+
+Specifies whether, by default, API methods calls should authenticate.
+
+=item datetime_parser
+
+Specifies the Date::Time::Format derived parser to use for parsing and
+formatting date strings for the API being defined.
+
 =over 4
 
 =item description
